@@ -30,9 +30,7 @@ class H2OGAMClassifier(H2OGAM):
                  betaConstraints=None,
                  gamCols=None,
                  columnsToCategorical=[],
-                 keepBinaryModels=False,
                  withContributions=False,
-                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
                  namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
@@ -43,6 +41,7 @@ class H2OGAMClassifier(H2OGAM):
                  convertUnknownCategoricalLevelsToNa=False,
                  splitRatio=1.0,
                  withStageResults=False,
+                 withDetailedPredictionCol=True,
                  seed=-1,
                  family="AUTO",
                  tweedieVariancePower=0.0,
@@ -51,7 +50,6 @@ class H2OGAMClassifier(H2OGAM):
                  solver="AUTO",
                  alphaValue=None,
                  lambdaValue=None,
-                 startval=None,
                  lambdaSearch=False,
                  earlyStopping=True,
                  nlambdas=-1,
@@ -76,12 +74,9 @@ class H2OGAMClassifier(H2OGAM):
                  computePValues=False,
                  removeCollinearCols=False,
                  numKnots=None,
-                 splineOrders=None,
                  scale=None,
                  bs=None,
                  keepGamCols=False,
-                 standardizeTpGamCols=False,
-                 scaleTpPenaltyMat=False,
                  knotIds=None,
                  modelId=None,
                  nfolds=0,
@@ -100,14 +95,12 @@ class H2OGAMClassifier(H2OGAM):
                  stoppingMetric="AUTO",
                  stoppingTolerance=0.001,
                  customMetricFunc=None,
-                 exportCheckpointsDir=None,
-                 aucType="AUTO"):
+                 exportCheckpointsDir=None):
         Initializer.load_sparkling_jar()
         super(H2OGAM, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.classification.H2OGAMClassifier", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
-        kwargs = self._updateInitKwargs(kwargs)
 
         if 'interactionPairs' in kwargs:
             warn("Interaction pairs are not supported!")
