@@ -1,5 +1,6 @@
 # -*- encoding: utf-8 -*-
 from __future__ import absolute_import, division, print_function, unicode_literals
+from h2o.utils.compatibility import *  # NOQA
 
 import os
 import traceback
@@ -9,7 +10,6 @@ from h2o.base import Keyed
 from h2o.exceptions import H2OValueError
 from h2o.job import H2OJob
 from h2o.utils.metaclass import BackwardsCompatible, Deprecated as deprecated, h2o_meta
-from h2o.utils.compatibility import *  # NOQA
 from h2o.utils.compatibility import viewitems
 from h2o.utils.shared_utils import can_use_pandas
 from h2o.utils.typechecks import I, assert_is_type, assert_satisfies, Enum, is_type
@@ -28,6 +28,7 @@ class ModelBase(h2o_meta(Keyed)):
         self._id = None
         self._model_json = None
         self._metrics_class = None
+        self._metrics_class_valid = None
         self._is_xvalidated = False
         self._xval_keys = None
         self._parms = {}  # internal, for object recycle
@@ -68,7 +69,8 @@ class ModelBase(h2o_meta(Keyed)):
         params = {}
         for p in self.parms:
             params[p] = {"default": self.parms[p]["default_value"],
-                         "actual": self.parms[p]["actual_value"]}
+                         "actual": self.parms[p]["actual_value"],
+                         "input": self.parms[p]["input_value"]}
         return params
 
 
@@ -385,7 +387,7 @@ class ModelBase(h2o_meta(Keyed)):
                 if mm["frame"] is not None and mm["frame"]["name"] == test_data.frame_id:
                     raw_metrics = mm
                     break
-            return self._metrics_class(raw_metrics, algo=self._model_json["algo"])
+            return self._metrics_class_valid(raw_metrics, algo=self._model_json["algo"])
 
 
     def scoring_history(self):
@@ -414,6 +416,50 @@ class ModelBase(h2o_meta(Keyed)):
         print("No actual number of trees for this model")    
 
 
+    def feature_interaction(self, max_interaction_depth = 100, max_tree_depth = 100, max_deepening = -1):
+        """
+        Feature interactions and importance, leaf statistics and split value histograms in a tabular form.
+        Available for XGBoost and GBM.
+
+        Metrics:
+        Gain - Total gain of each feature or feature interaction.
+        FScore - Amount of possible splits taken on a feature or feature interaction.
+        wFScore - Amount of possible splits taken on a feature or feature interaction weighed by 
+        the probability of the splits to take place.
+        Average wFScore - wFScore divided by FScore.
+        Average Gain - Gain divided by FScore.
+        Expected Gain - Total gain of each feature or feature interaction weighed by the probability to gather the gain.
+        Average Tree Index
+        Average Tree Depth
+
+        :param max_interaction_depth: Upper bound for extracted feature interactions depth. Defaults to 100.
+        :param max_tree_depth: Upper bound for tree depth. Defaults to 100.
+        :param max_deepening: Upper bound for interaction start deepening (zero deepening => interactions 
+        starting at root only). Defaults to -1.
+
+        :examples:
+        >>> boston = h2o.import_file("https://s3.amazonaws.com/h2o-public-test-data/smalldata/gbm_test/BostonHousing.csv")
+        >>> predictors = boston.columns[:-1]
+        >>> response = "medv"
+        >>> boston['chas'] = boston['chas'].asfactor()
+        >>> train, valid = boston.split_frame(ratios=[.8])
+        >>> boston_xgb = H2OXGBoostEstimator(seed=1234)
+        >>> boston_xgb.train(y=response, x=predictors, training_frame=train)
+        >>> feature_interactions = boston_xgb.feature_interaction()
+        """
+        supported_algos = ['gbm', 'xgboost']
+        if self._model_json["algo"] in supported_algos:
+            kwargs = {}
+            kwargs["model_id"] = self.model_id
+            kwargs["max_interaction_depth"] = max_interaction_depth
+            kwargs["max_tree_depth"] = max_tree_depth
+            kwargs["max_deepening"] = max_deepening
+            
+            json = h2o.api("POST /3/FeatureInteraction", data=kwargs)
+            
+            return json['feature_interaction']
+        print("No calculation available for this model")
+
     def cross_validation_metrics_summary(self):
         """
         Retrieve Cross-Validation Metrics Summary.
@@ -482,23 +528,9 @@ class ModelBase(h2o_meta(Keyed)):
         :returns: A list or Pandas DataFrame.
         """
         model = self._model_json["output"]
-        if self.algo=='glm' or self.algo=='gam' or "variable_importances" in list(model.keys()) and model["variable_importances"]:
-            if self.algo=='glm' or self.algo=='gam':
-                tempvals = model["standardized_coefficient_magnitudes"].cell_values
-                maxVal = 0
-                sum=0
-                for item in tempvals:
-                    sum=sum+item[1]
-                    if item[1]>maxVal:
-                        maxVal = item[1]
-                vals = []
-                for item in tempvals:
-                    tempT = (item[0], item[1], item[1]/maxVal, item[1]/sum)
-                    vals.append(tempT)
-                header = ["variable", "relative_importance", "scaled_importance", "percentage"]
-            else:
-                vals = model["variable_importances"].cell_values
-                header = model["variable_importances"].col_header
+        if "variable_importances" in list(model.keys()) and model["variable_importances"]:
+            vals = model["variable_importances"].cell_values
+            header = model["variable_importances"].col_header
                 
             if use_pandas and can_use_pandas():
                 import pandas
@@ -853,7 +885,8 @@ class ModelBase(h2o_meta(Keyed)):
         m = {}
         for k, v in viewitems(tm): m[k] = None if v is None else v.gini()
         return list(m.values())[0] if len(m) == 1 else m
-
+    
+    
     def aucpr(self, train=False, valid=False, xval=False):
         """
         Get the aucPR (Area Under PRECISION RECALL Curve).
@@ -1050,8 +1083,7 @@ class ModelBase(h2o_meta(Keyed)):
 
         else:  # algo is not glm, deeplearning, drf, gbm, xgboost
             raise H2OValueError("Plotting not implemented for this type of model")
-        if not server: plt.show()
-
+        if not server: plt.show()        
 
     def partial_plot(self, data, cols=None, destination_key=None, nbins=20, weight_column=None,
                      plot=True, plot_stddev = True, figsize=(7, 10), server=False, include_na=False, user_splits=None,
@@ -1118,7 +1150,7 @@ class ModelBase(h2o_meta(Keyed)):
                 raise H2OValueError("Column %s does not exist in the data frame" % weight_column)
             weight_column = data.names.index(weight_column)
         
-        if row_index:
+        if row_index is not None:
             if not isinstance(row_index, int):
                 raise H2OValueError("Row index should be of type int.")
         else:
@@ -1153,7 +1185,7 @@ class ModelBase(h2o_meta(Keyed)):
 
         # Plot partial dependence plots using matplotlib
         self.__generate_partial_plots(num_1dpdp, num_2dpdp, plot, server, pps, figsize, col_pairs_2dpdp, data, nbins,
-                                      kwargs["user_cols"], kwargs["num_user_splits"], plot_stddev, cols, save_to_file, targets)
+                                      kwargs["user_cols"], kwargs["num_user_splits"], plot_stddev, cols, save_to_file, row_index, targets)
         return pps
 
     def __generate_user_splits(self, user_splits, data, kwargs):
@@ -1207,7 +1239,7 @@ class ModelBase(h2o_meta(Keyed)):
             kwargs["num_user_splits"] = None
 
     def __generate_partial_plots(self, num_1dpdp, num_2dpdp, plot, server, pps, figsize, col_pairs_2dpdp, data, nbins,
-                                 user_cols, user_num_splits, plot_stddev, cols, save_to_file, targets):
+                                 user_cols, user_num_splits, plot_stddev, cols, save_to_file, row_index, targets):
         # Plot partial dependence plots using matplotlib
         to_fig = num_1dpdp + num_2dpdp
         if plot and to_fig > 0:     # plot 1d pdp for now
@@ -1230,14 +1262,14 @@ class ModelBase(h2o_meta(Keyed)):
                     if axes_3d is None or cm is None or plt is None:    # quit if cannot find toolbox
                         break
                     fig_plotted = self.__plot_2d_pdp(fig, col_pairs_2dpdp, gxs, num_1dpdp, data, pps[i], nbins,
-                                                     user_cols, user_num_splits, plot_stddev, cm, i)                  
+                                                     user_cols, user_num_splits, plot_stddev, cm, i, row_index)                  
                 else:  # plot 1D pdp
                     col = cols[i]
                     if targets is None or target:
-                        fig_plotted = self.__plot_1d_pdp(col, i, data, pps[i], fig, gxs, plot_stddev, target)
+                        fig_plotted = self.__plot_1d_pdp(col, i, data, pps[i], fig, gxs, plot_stddev, row_index, target)
                     else:
                         fig_plotted = self.__plot_1d_pdp_multinomial(col, i, data, pps, data_index, fig, gxs, cm, 
-                                                                     plot_stddev, targets)
+                                                                     plot_stddev, row_index, targets)
                         data_index = data_index + len(targets)
             if fig_plotted:
                 fig.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)
@@ -1248,7 +1280,7 @@ class ModelBase(h2o_meta(Keyed)):
                 plt.savefig(save_to_file)
 
     def __plot_2d_pdp(self, fig, col_pairs_2dpdp, gxs, num_1dpdp, data, pp, nbins, user_cols, user_num_splits, 
-                      plot_stddev, cm, i):
+                      plot_stddev, cm, i, row_index):
         ax = fig.add_subplot(gxs[i], projection='3d')
         col_pairs = col_pairs_2dpdp[i-num_1dpdp]
         x = self.__grab_values(pp, 0, data, col_pairs[0], ax) # change to numpy 2d_array
@@ -1273,19 +1305,22 @@ class ModelBase(h2o_meta(Keyed)):
         ax.set_ylim(min(y), max(y))
         ax.set_zlabel('Partial dependence')
         title = '2D partial dependence plot for '+col_pairs[0] + ' and '+col_pairs[1]
+        if row_index >= 0:
+            title += ' and row index {}'.format(row_index)
         ax.set_title(title)
         return True
     
-    def __plot_1d_pdp(self, col, i, data, pp, fig, gxs, plot_stddev, target=None):
+    def __plot_1d_pdp(self, col, i, data, pp, fig, gxs, plot_stddev, row_index, target=None):
         cat = data[col].isfactor()[0]
         axs = fig.add_subplot(gxs[i])
-        self.__set_axs_1d(axs, plot_stddev, cat, pp, col, target) 
+        self.__set_axs_1d(axs, plot_stddev, cat, pp, col, row_index, target) 
         return True
     
-    def __plot_1d_pdp_multinomial(self, col, i, data, pps, data_start_index, fig, gxs, cm, plot_stddev, targets):
+    def __plot_1d_pdp_multinomial(self, col, i, data, pps, data_start_index, fig, gxs, cm, plot_stddev, row_index, 
+                                    targets):
         cat = data[col].isfactor()[0]
         axs = fig.add_subplot(gxs[i])
-        self.__set_axs_1d_multinomial(axs, cm, plot_stddev, cat, pps, data_start_index, col, targets)
+        self.__set_axs_1d_multinomial(axs, cm, plot_stddev, cat, pps, data_start_index, col, row_index, targets)
         return True
         
     # change x, y, z to be 2-D numpy arrays in order to plot it.
@@ -1310,7 +1345,7 @@ class ModelBase(h2o_meta(Keyed)):
         cat = data[col].isfactor()[0]
         if cat:
             labels = pp[index]
-            uniqueL =list(set(labels))
+            uniqueL = list(set(labels))
             x = range(len(uniqueL))
             xlab = [None]*len(uniqueL)
             for ind in range(len(uniqueL)):
@@ -1326,14 +1361,13 @@ class ModelBase(h2o_meta(Keyed)):
                 axs.set_xticklabels(xlab)
             else:   # y-axis
                 axs.set_yticks(x)
-                axs.set_yticklabels(labels)
+                axs.set_yticklabels(xlab)
             axs.margins(0.2) 
-
             return xext
         else:
             return pp[index]
         
-    def __set_axs_1d(self, axs, plot_stddev, cat, pp, col, target):
+    def __set_axs_1d(self, axs, plot_stddev, cat, pp, col, row_index, target):
         pp_start_index = 0
         x = pp[pp_start_index]
         y = pp[pp_start_index+1]
@@ -1347,7 +1381,7 @@ class ModelBase(h2o_meta(Keyed)):
             x = range(len(labels))
             fmt = "o"
             axs.set_xticks(x)
-            axs.set_xticklabels(labels)
+            axs.set_xticklabels(labels, rotation=45)
             axs.set_xlim(min(x) - 0.2, max(x) + 0.2)
         if plot_stddev:
             std = pp[pp_start_index+2]
@@ -1362,17 +1396,18 @@ class ModelBase(h2o_meta(Keyed)):
         else:
             axs.plot(x, y, fmt, label=target)
             axs.set_ylim(min(y) - 0.2 * abs(min(y)), max(y) + 0.2 * abs(max(y)))
+        title = "Partial Dependence Plot for {}".format(col)
         if target:
-            axs.set_title("Partial Dependence Plot for {} and class {}".format(col, target))
-            axs.legend()
-        else:
-            axs.set_title("Partial Dependence Plot for {}".format(col))
+            title += " and class {}".format(target)
+        if row_index >= 0:
+            title += " and row index {}".format(row_index)
+        axs.set_title(title)
         axs.set_xlabel(pp.col_header[pp_start_index])
         axs.set_ylabel(pp.col_header[pp_start_index+1])
         axs.xaxis.grid()
         axs.yaxis.grid()
         
-    def __set_axs_1d_multinomial(self, axs, cm, plot_stddev, cat, pps, data_start_index, col, targets):
+    def __set_axs_1d_multinomial(self, axs, cm, plot_stddev, cat, pps, data_start_index, col, row_index, targets):
         pp_start_index = 0
         pp = pps[data_start_index]
         x = pp[pp_start_index]
@@ -1388,7 +1423,7 @@ class ModelBase(h2o_meta(Keyed)):
             labels = pp[pp_start_index]
             x = range(len(labels))
             axs.set_xticks(x)
-            axs.set_xticklabels(labels)
+            axs.set_xticklabels(labels, rotation=45)
             fmt = "o"
             axs.set_xlim(min(x) - 0.2, max(x) + 0.2)
         else:
@@ -1419,7 +1454,10 @@ class ModelBase(h2o_meta(Keyed)):
         else:
             axs.set_ylim(min_y - 0.2 * abs(min_y), max_y + 0.2 * abs(max_y))
         axs.legend()
-        axs.set_title("Partial Dependence Plot for {} and classes \n {}".format(col, ', '.join(targets)))
+        title = "Partial Dependence Plot for {} and classes \n {}".format(col, ', '.join(targets))
+        if row_index >= 0:
+            title += " and row index {}".format(row_index)
+        axs.set_title(title)
         axs.xaxis.grid()
         axs.yaxis.grid()
         
@@ -1707,12 +1745,11 @@ class ModelBase(h2o_meta(Keyed)):
 
 
 
-
 def _get_matplotlib_pyplot(server):
     try:
         # noinspection PyUnresolvedReferences
         import matplotlib
-        if server: matplotlib.use("Agg", warn=False)
+        if server: matplotlib.use("Agg")
         # noinspection PyUnresolvedReferences
         import matplotlib.pyplot as plt
         return plt
